import numpy, cv2, cam_model

def calcHomography(objp, imgp):
    numPts = objp.shape[0]
    A = numpy.zeros((numPts*2, 9), float)
    imgp = imgp.reshape(numPts,2)
    for i in range(numPts):
        A[i*2+0,0:2] = objp[i,0:2]
        A[i*2+0,2] = 1.0
        A[i*2+0,6:9] = -imgp[i,0]*A[i*2+0,0:3]
        A[i*2+1,3:5] = objp[i,0:2]
        A[i*2+1,5] = 1.0
        A[i*2+1,6:9] = -imgp[i,1]*A[i*2+1,3:6]
    U, w, Vt = numpy.linalg.svd(A)
    return Vt[8,:].reshape(3,3)

if __name__ == "__main__":

    cap = cv2.VideoCapture(1)
    #cap.set(cv2.cv.CV_CAP_PROP_FRAME_WIDTH, 640)
    #cap.set(cv2.cv.CV_CAP_PROP_FRAME_HEIGHT, 480)

    K = numpy.loadtxt('K.txt')
    dist = numpy.loadtxt('distCoef.txt')

    patw, path = 7, 6
    objp = numpy.zeros((patw*path, 3), numpy.float32)
    for i in range(patw*path):
        objp[i,:2] = numpy.array([i % patw, i / patw])

    while 1:
        stat, image = cap.read(0)

        image = cv2.resize(image, (1920/2, 1024/2))

        retval, centers = cv2.findCirclesGrid(image, (patw, path))

        if retval == True:
            H = calcHomography(objp, centers) # H = K[r0,r1,t]
            Q = numpy.linalg.inv(K).dot(H)   # Q=K^{-1}H
            Q = Q*numpy.sign(numpy.linalg.det(Q))
            R = Q / (numpy.linalg.norm(Q[:,0])) # Normalize
            t = R[:,2].reshape(3,1).copy() # Deep copy
            R[:,2] = numpy.cross(R[:,0],R[:,1]) # r2=r0xr1
            print 'R=\n', R, '\nt=\n', t

            cam_model.draw_cube(image, K, numpy.array([[0.0,0.0,0.0]], float).T,\
                                3.0, R, t)

        cv2.imshow('Camera', image)
        key = cv2.waitKey(10)

        if key == 0x1b: # ESC
            break

    cap.release()
    cv2.destroyAllWindows()




